@
@ GCat's Master System Emulator
@ Copyright (c) 2012 James Perry
@
@ This program is free software: you can redistribute it and/or modify
@ it under the terms of the GNU General Public License as published by
@ the Free Software Foundation, either version 3 of the License, or
@ (at your option) any later version.
@
@ This program is distributed in the hope that it will be useful,
@ but WITHOUT ANY WARRANTY; without even the implied warranty of
@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@ GNU General Public License for more details.
@
@ You should have received a copy of the GNU General Public License
@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
@

@
@ All IX/IY instructions (apart from the CB-prefixed ones) are implemented
@ in here.
@
@ As much code as possible is shared between IX and IY. The insnfd and
@ insndd handlers at the start read the correct index register into r8
@ and the rest of the code uses r8. Instructions that modify IX/IY
@ should end with the END_INSN_IX macro which stores r8 back in the
@ right place. Others can use END_INSN as normal.
@
insnfd: ldrh	r8, [r0, #26]		@ read IY
	mov	r2, #1
	strb	r2, [r0, #74]
	b	insnix

insndd:	ldrh	r8, [r0, #24]		@ read IX
	mov	r2, #0
	strb	r2, [r0, #74]
	
insnix: @ read second byte of instruction
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)

	@ increment R again
	add	r4, r4, #16777216

	@ lookup instruction and jump to it
	ldr	r2,[r0, #64]
	ldr	pc,[r2, r3, lsl #2]

	.global	ddinsn_table
ddinsn_table:
	.word	insn00, insn01, insn02, insn03
	.word	insn04, insn05, insn06, insn07
	.word	insn08, ixinsn09, insn0a, insn0b
	.word	insn0c, insn0d, insn0e, insn0f
	.word	insn10, insn11, insn12, insn13
	.word	insn14, insn15, insn16, insn17
	.word	insn18, ixinsn19, insn1a, insn1b
	.word	insn1c, insn1d, insn1e, insn1f
	.word	insn20, ixinsn21, ixinsn22, ixinsn23
	.word	ixinsn24, ixinsn25, ixinsn26, insn27
	.word	insn28, ixinsn29, ixinsn2a, ixinsn2b
	.word	ixinsn2c, ixinsn2d, ixinsn2e, insn2f
	.word	insn30, insn31, insn32, insn33
	.word	ixinsn34, ixinsn35, ixinsn36, insn37
	.word	insn38, ixinsn39, insn3a, insn3b
	.word	insn3c, insn3d, insn3e, insn3f

	.word	insn40, insn41, insn42, insn43
	.word	ixinsn44, ixinsn45, ixinsn46, insn47
	.word	insn48, insn49, insn4a, insn4b
	.word	ixinsn4c, ixinsn4d, ixinsn4e, insn4f
	.word	insn50, insn51, insn52, insn53
	.word	ixinsn54, ixinsn55, ixinsn56, insn57
	.word	insn58, insn59, insn5a, insn5b
	.word	ixinsn5c, ixinsn5d, ixinsn5e, insn5f
	.word	ixinsn60, ixinsn61, ixinsn62, ixinsn63
	.word	ixinsn64, ixinsn65, ixinsn66, ixinsn67
	.word	ixinsn68, ixinsn69, ixinsn6a, ixinsn6b
	.word	ixinsn6c, ixinsn6d, ixinsn6e, ixinsn6f
	.word	ixinsn70, ixinsn71, ixinsn72, ixinsn73
	.word	ixinsn74, ixinsn75, insn76, ixinsn77
	.word	insn78, insn79, insn7a, insn7b
	.word	ixinsn7c, ixinsn7d, ixinsn7e, insn7f

	.word	insn80, insn81, insn82, insn83
	.word	ixinsn84, ixinsn85, ixinsn86, insn87
	.word	insn88, insn89, insn8a, insn8b
	.word	ixinsn8c, ixinsn8d, ixinsn8e, insn8f
	.word	insn90, insn91, insn92, insn93
	.word	ixinsn94, ixinsn95, ixinsn96, insn97
	.word	insn98, insn99, insn9a, insn9b
	.word	ixinsn9c, ixinsn9d, ixinsn9e, insn9f
	.word	insna0, insna1, insna2, insna3
	.word	ixinsna4, ixinsna5, ixinsna6, insna7
	.word	insna8, insna9, insnaa, insnab
	.word	ixinsnac, ixinsnad, ixinsnae, insnaf
	.word	insnb0, insnb1, insnb2, insnb3
	.word	ixinsnb4, ixinsnb5, ixinsnb6, insnb7
	.word	insnb8, insnb9, insnba, insnbb
	.word	ixinsnbc, ixinsnbd, ixinsnbe, insnbf

	.word	insnc0, insnc1, insnc2, insnc3
	.word	insnc4, insnc5, insnc6, insnc7
	.word	insnc8, insnc9, insnca, ixinsncb
	.word	insncc, insncd, insnce, insncf
	.word	insnd0, insnd1, insnd2, insnd3
	.word	insnd4, insnd5, insnd6, insnd7
	.word	insnd8, insnd9, insnda, insndb
	.word	insndc, ixnop, insnde, insndf
	.word	insne0, ixinsne1, insne2, ixinsne3
	.word	insne4, ixinsne5, insne6, insne7
	.word	insne8, ixinsne9, insnea, insneb
	.word	insnec, ixnop, insnee, insnef
	.word	insnf0, insnf1, insnf2, insnf3
	.word	insnf4, insnf5, insnf6, insnf7
	.word	insnf8, ixinsnf9, insnfa, insnfb
	.word	insnfc, ixnop, insnfe, insnff

ixnop:	END_INSN(#8)

	@ add ix,bc
ixinsn09:
	ADD16(r8, r5, r3, r2)
	END_INSN_IX(#15)
	
	@ add ix,de
ixinsn19:
	ADD16(r8, r6, r3, r2)
	END_INSN_IX(#15)
	
	@ ld ix,NN
ixinsn21:
	READ_BYTE(r8, r9, r3, r2)
	INC16(r9)
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#14)
	
	@ ld (NN),ix
ixinsn22:
	READ_BYTE(r12, r9, r3, r2)
	INC16(r9)
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)
	orr	r12, r12, r3, lsl #8
	WRITE_BYTE(r8, r12, r3, r2)
	INC16(r12)
	mov	r8, r8, ror #8
	WRITE_BYTE(r8, r12, r3, r2)
	END_INSN(#20)
	
	@ inc ix
ixinsn23:
	INC16(r8)
	END_INSN_IX(#10)
	
	@ inc ixh
ixinsn24:
	INC8_HI(r8, r2)
	END_INSN_IX(#8)
	
	@ dec ixh
ixinsn25:
	DEC8_HI(r8, r2)
	END_INSN_IX(#8)
	
	@ ld ixh,N
ixinsn26:
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)
	and	r8, r8, #0xff
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#11)
	
	@ add ix,ix
ixinsn29:
	ADD16(r8, r8, r3, r2)
	END_INSN_IX(#15)
	
	@ ld ix,(NN)
ixinsn2a:
	READ_BYTE(r12, r9, r3, r2)
	INC16(r9)
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)
	orr	r12, r12, r3, lsl #8
	READ_BYTE(r8, r12, r3, r2)
	INC16(r12)
	READ_BYTE(r3, r12, r3, r2)
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#20)

	@ dec ix
ixinsn2b:
	DEC16(r8)
	END_INSN_IX(#10)
	
	@ inc ixl
ixinsn2c:
	INC8_LO(r8, r2)
	END_INSN_IX(#8)
	
	@ dec ixl
ixinsn2d:
	DEC8_LO(r8, r2)
	END_INSN_IX(#8)
	
	@ ld ixl,N
ixinsn2e:
	READ_BYTE(r3, r9, r3, r2)
	INC16(r9)
	and	r8, r8, #0xff00
	orr	r8, r8, r3
	END_INSN_IX(#11)
	
	@ inc (ix+d)
ixinsn34:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	INC8_LO(r12, r2)
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#23)
	
	@ dec (ix+d)
ixinsn35:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	DEC8_LO(r12, r2)
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#23)

	@ ld (ix+d),N
ixinsn36:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r9, r3, r2)
	INC16(r9)
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#19)
	
	@ add ix,sp
ixinsn39:
	ldrh	r12,[r0, #20]
	ADD16(r8, r12, r3, r2)
	END_INSN_IX(#15)

	@ ld b,ixh
ixinsn44:
	and	r3, r8, #0xff00
	and	r5, r5, #0x00ff
	orr	r5, r5, r3
	END_INSN(#8)
	@ ld b,ixl
ixinsn45:
	and	r5, r5, #0x00ff
	and	r3, r8, #0x00ff
	orr	r5, r5, r3, lsl #8
	END_INSN(#8)
	@ ld b,(ix+d)
ixinsn46:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r5, r5, #0x00ff
	orr	r5, r5, r3, lsl #8
	END_INSN(#19)
	
	@ ld c,ixh
ixinsn4c:
	and	r5, r5, #0xff00
	orr	r5, r5, r8, lsr #8
	END_INSN(#8)
	@ ld c,ixl
ixinsn4d:
	and	r5, r5, #0xff00
	and	r3, r8, #0xff
	orr	r5, r5, r3
	END_INSN(#8)
	@ ld c,(ix+d)
ixinsn4e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r5, r5, #0xff00
	orr	r5, r5, r3
	END_INSN(#19)

	@ ld d,ixh
ixinsn54:
	and	r6, r6, #0xff
	and	r3, r8, #0xff00
	orr	r6, r6, r3
	END_INSN(#8)
	@ ld d,ixl
ixinsn55:
	and	r6, r6, #0xff
	and	r3, r8, #0xff
	orr	r6, r6, r3, lsl #8
	END_INSN(#8)
	@ ld d,(ix+d)
ixinsn56:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r6, r6, #0x00ff
	orr	r6, r6, r3, lsl #8
	END_INSN(#19)
	
	@ ld e,ixh
ixinsn5c:
	and	r6, r6, #0xff00
	orr	r6, r6, r8, lsr #8
	END_INSN(#8)
	@ ld e,ixl
ixinsn5d:
	and	r6, r6, #0xff00
	and	r3, r8, #0x00ff
	orr	r6, r6, r3
	END_INSN(#8)
	@ ld e,(ix+d)
ixinsn5e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r6, r6, #0xff00
	orr	r6, r6, r3
	END_INSN(#19)

	@ ld ixh,b
ixinsn60:
	and	r8, r8, #0xff
	and	r3, r5, #0xff00
	orr	r8, r8, r3
	END_INSN_IX(#8)
	@ ld ixh,c
ixinsn61:
	and	r8, r8, #0xff
	and	r3, r5, #0xff
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#8)
	@ ld ixh,d
ixinsn62:
	and	r8, r8, #0xff
	and	r3, r6, #0xff00
	orr	r8, r8, r3
	END_INSN_IX(#8)
	@ ld ixh,e
ixinsn63:
	and	r8, r8, #0xff
	and	r3, r6, #0xff
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#8)
	@ ld ixh,ixh
ixinsn64:
	END_INSN(#8)
	@ ld ixh,ixl
ixinsn65:
	and	r8, r8, #0xff
	orr	r8, r8, r8, lsl #8
	END_INSN_IX(#8)
	@ ld h,(ix+d)
ixinsn66:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r7, r7, #0x00ff
	orr	r7, r7, r3, lsl #8
	END_INSN(#19)
	@ ld ixh,a
ixinsn67:
	and	r8, r8, #0xff
	and	r3, r4, #0xff
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#8)
	
	@ ld ixl,b
ixinsn68:
	and	r8, r8, #0xff00
	orr	r8, r8, r5, lsr #8
	END_INSN_IX(#8)
	@ ld ixl,c
ixinsn69:
	and	r8, r8, #0xff00
	and	r3, r5, #0xff
	orr	r8, r8, r3
	END_INSN_IX(#8)
	@ ld ixl,d
ixinsn6a:
	and	r8, r8, #0xff00
	orr	r8, r8, r6, lsr #8
	END_INSN_IX(#8)
	@ ld ixl,e
ixinsn6b:
	and	r8, r8, #0xff00
	and	r3, r6, #0xff
	orr	r8, r8, r3
	END_INSN_IX(#8)
	@ ld ixl,ixh
ixinsn6c:
	and	r8, r8, #0xff00
	orr	r8, r8, r8, lsr #8
	END_INSN_IX(#8)
	@ ld ixl,ixl
ixinsn6d:
	END_INSN(#8)
	@ ld l,(ix+d)
ixinsn6e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	and	r7, r7, #0xff00
	orr	r7, r7, r3
	END_INSN(#19)
	@ ld ixl,a
ixinsn6f:
	and	r8, r8, #0xff00
	and	r3, r4, #0xff
	orr	r8, r8, r3
	END_INSN_IX(#8)

	@ ld (ix+d),b
ixinsn70:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	mov	r12, r5, lsr #8
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),c
ixinsn71:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	WRITE_BYTE(r5, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),d
ixinsn72:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	mov	r12, r6, lsr #8
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),e
ixinsn73:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	WRITE_BYTE(r6, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),h
ixinsn74:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	mov	r12, r7, lsr #8
	WRITE_BYTE(r12, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),l
ixinsn75:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	WRITE_BYTE(r7, r8, r3, r2)
	END_INSN(#19)
	@ ld (ix+d),a
ixinsn77:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	WRITE_BYTE(r4, r8, r3, r2)
	END_INSN(#19)
	@ ld a,ixh
ixinsn7c:
	bic	r4, r4, #0xff
	orr	r4, r4, r8, lsr #8
	END_INSN(#8)
	@ ld a,ixl
ixinsn7d:
	bic	r4, r4, #0xff
	and	r3, r8, #0xff
	orr	r4, r4, r3
	END_INSN(#8)
	@ ld a,(ix+d)
ixinsn7e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r3, r8, r3, r2)
	bic	r4, r4, #0xff
	orr	r4, r4, r3
	END_INSN(#19)

	@ add a,ixh
ixinsn84:
	mov	r12, r8, lsr #8
	ADD8(r12, r2, r3)
	END_INSN(#8)
	@ add a,ixl
ixinsn85:
	and	r12, r8, #0xff
	ADD8(r12, r2, r3)
	END_INSN(#8)
	@ add a,(ix+d)
ixinsn86:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	ADD8(r12, r2, r3)
	END_INSN(#19)
	@ adc a,ixh
ixinsn8c:
	mov	r12, r8, lsr #8
	ADC8(r12, r2, r3)
	END_INSN(#8)
	@ adc a,ixl
ixinsn8d:
	and	r12, r8, #0xff
	ADC8(r12, r2, r3)
	END_INSN(#8)
	@ adc a,(ix+d)
ixinsn8e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	ADC8(r12, r2, r3)
	END_INSN(#19)

	@ sub ixh
ixinsn94:
	mov	r12, r8, lsr #8
	SUB8(r12, r2, r3)
	END_INSN(#8)
	@ sub ixl
ixinsn95:
	and	r12, r8, #0xff
	SUB8(r12, r2, r3)
	END_INSN(#8)
	@ sub (ix+d)
ixinsn96:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	SUB8(r12, r2, r3)
	END_INSN(#19)
	@ sbc a,ixh
ixinsn9c:
	mov	r12, r8, lsr #8
	SBC8(r12, r2, r3)
	END_INSN(#8)
	@ sbc a,ixl
ixinsn9d:
	and	r12, r8, #0xff
	SBC8(r12, r2, r3)
	END_INSN(#8)
	@ sbc a,(ix+d)
ixinsn9e:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	SBC8(r12, r2, r3)
	END_INSN(#19)

	@ and ixh
ixinsna4:
	mov	r12, r8, lsr #8
	AND8(r12, r2, r3)
	END_INSN(#8)
	@ and ixl
ixinsna5:
	and	r12, r8, #0xff
	AND8(r12, r2, r3)
	END_INSN(#8)
	@ and (ix+d)
ixinsna6:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	AND8(r12, r2, r3)
	END_INSN(#19)
	@ xor ixh
ixinsnac:
	mov	r12, r8, lsr #8
	XOR8(r12, r2, r3)
	END_INSN(#8)
	@ xor ixl
ixinsnad:
	and	r12, r8, #0xff
	XOR8(r12, r2, r3)
	END_INSN(#8)
	@ xor (ix+d)
ixinsnae:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	XOR8(r12, r2, r3)
	END_INSN(#19)

	@ or ixh
ixinsnb4:
	mov	r12, r8, lsr #8
	OR8(r12, r2, r3)
	END_INSN(#8)
	@ or ixl
ixinsnb5:
	and	r12, r8, #0xff
	OR8(r12, r2, r3)
	END_INSN(#8)
	@ or (ix+d)
ixinsnb6:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	OR8(r12, r2, r3)
	END_INSN(#19)
	@ cp ixh
ixinsnbc:
	mov	r12, r8, lsr #8
	CP8(r12, r2, r3)
	END_INSN(#8)
	@ cp ixl
ixinsnbd:
	and	r12, r8, #0xff
	CP8(r12, r2, r3)
	END_INSN(#8)
	@ cp (ix+d)
ixinsnbe:
	READ_SIGNED_BYTE(r3, r9, r3, r2)
	INC16(r9)
	add	r8, r8, r3
	bic	r8, r8, #0xff000000
	bic	r8, r8, #0x00ff0000
	READ_BYTE(r12, r8, r3, r2)
	CP8(r12, r2, r3)
	END_INSN(#19)

	@ pop ix
ixinsne1:
	ldrh	r12,[r0, #20]
	READ_BYTE(r8, r12, r3, r2)
	INC16(r12)
	READ_BYTE(r3, r12, r3, r2)
	INC16(r12)
	strh	r12,[r0, #20]
	orr	r8, r8, r3, lsl #8
	END_INSN_IX(#14)
	
	@ ex (sp),ix
ixinsne3:
	ldrh	r12,[r0, #20]
	strh	r5,[r0, #14]
	READ_BYTE(r5, r12, r3, r2)
	WRITE_BYTE(r8, r12, r3, r2)
	INC16(r12)
	READ_BYTE(r3, r12, r3, r2)
	orr	r5, r5, r3, lsl #8
	mov	r8, r8, ror #8
	WRITE_BYTE(r8, r12, r3, r2)
	mov	r8, r5
	ldrh	r5,[r0, #14]
	END_INSN_IX(#23)	
	
	@ push ix
ixinsne5:
	ldrh	r12,[r0, #20]
	DEC16(r12)
	mov	r8, r8, ror #8
	WRITE_BYTE(r8, r12, r3, r2)
	DEC16(r12)
	mov	r8, r8, ror #24
	WRITE_BYTE(r8, r12, r3, r2)
	strh	r12,[r0, #20]
	END_INSN(#15)
	
	@ jp (ix)
ixinsne9:
	mov	r9, r8
	END_INSN(#8)

	@ ld sp,ix
ixinsnf9:
	strh	r8,[r0, #20]
	END_INSN(#10)
